import 'dart:async';

import 'package:flutter/services.dart';
import 'linkv_rtm_callback.dart';
import 'linkv_api_defines.dart';
import 'linkv_video_callback.dart';

class RtmFlutterPlugin {
  static const MethodChannel _channel =
      const MethodChannel('rtm_flutter_plugin');

  static LinkVVideoCallback _videoCallback;
  static LVIMEventCallback _rtmCallback;

  /// SDK 内部版本号，汇报给 linkv 时可以提供该版本号
  static Future<String> getSdkVersion() async {
    final String version = await _channel.invokeMethod("getSdkVersion");
    return version;
  }

  /// SDK 鉴权
  /// @param appId 向LinkV申请的appId
  /// @param appSign SDK签名字符串
  /// @param isTestEnv  YES 为使用测试环境，NO 为不使用测试环境（默认为 NO）
  /// @param isInternationalEnv YES 为国际版本，NO 为国内版本（默认为 NO）
  static Future<int> initSDK(String appId, String appSign, bool isDebugEnv,
      bool isInternationalEnv) async {
    final int result = await _channel.invokeMethod("initSDK", {
      'rtcAppID': appId,
      'rtcAppKey': appSign,
      'imAppID': appId,
      'imAppKey': appSign,
      'sdkType': 3,
      'isDebugEnv': isDebugEnv,
      'isInternationalEnv': isInternationalEnv
    });
    return result;
  }

  // 设置是否打开日志
  static void setLogOpen(bool isOpen) {
    _channel.invokeMethod("setLogOpen", {"isOpen": isOpen});
  }

  /// 推流参数配置
  ///
  /// @param videoConfigLevel 推流参数预设值
  static void setVideoConfig(int videoConfigLevel) async {
    await _channel
        .invokeMethod("setVideoConfig", {"videoConfigLevel": videoConfigLevel});
  }

  /// 登录，请使用自己的用户账号系统，确保userId唯一标识用户。
  /// @param 用户ID
  /// @return 0代表登录成功，-6代表已经是登录状态，其它代表失败。
  static Future<int> loginUser(String userId) async {
    return await _channel.invokeMethod("loginUser", {"userId": userId});
  }

  static void setIMToken(String uid, String token) async {
    if (token == null || token.length == 0 || uid == null || uid.length == 0)
      return;

    _channel.invokeMethod("setIMToken", {"userId": uid, "imToken": token});
  }

  /// 发送私信消息
  ///
  /// @param sub          媒体类型
  /// @param tid          接收者ID
  /// @param type         消息的自定义扩展类型，可作为消息标识，但不能传空或者空串。
  /// @param content      消息内容
  /// 返回0为成功，其它为错误码
  static Future<int> sendPrivateMessage(
      MessageSubType sub, String tid, String type, String content) async {
    int result = await _channel.invokeMethod("sendPrivateMessage", {
      "sub": sub.index,
      "tid": tid,
      "type": type,
      "content": content,
    });
    return result;
  }

  /// 发送房间消息
  ///
  /// @param targetId   房间Id
  /// @param msgContent 消息内容
  /// @param listener   发送结果监听
  static Future<int> sendRoomMessage(String targetId, String msgContent) async {
    return await _channel.invokeMethod("sendRoomMessage", {
      "targetId": targetId,
      "msgContent": msgContent,
    });
  }

  /// 卸载 SDK，销毁单例对象资源
  static void unInitSDK() async {
    await _channel.invokeMethod("unInitSDK");
  }

  /// 设置SDK事件监听
  static void setIMEventCallback(LVIMEventCallback callback) {
    _rtmCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  /// 设置房间事件监听
  static void setVideoCallback(LinkVVideoCallback callback) {
    _videoCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  /// 加入房间（音视频和聊天）
  /// @param roomId 房间 ID
  /// @param userId 用户 ID
  /// @param isHost 是否是主持人
  /// @param isOnlyAudio 是否以音频模式加入
  static void loginRoom(
      String roomId, String userId, bool isHost, bool isAudioOnly) async {
    await _channel.invokeMethod("loginRoom", {
      "roomId": roomId,
      "userId": userId,
      "isHost": isHost,
      "isAudioOnly": isAudioOnly
    });
  }

  static Future<Map> requestDebugToken(String userId, String username) async {
    if (userId == null || userId.length == 0) return null;
    return await _channel.invokeMethod(
        "requestDebugToken", {"userId": userId, "username": username});
  }

  /// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
  /// @param seconds 回调频率
  static void setPublishQualityMonitorCycle(int seconds) async {
    await _channel
        .invokeMethod("setPublishQualityMonitorCycle", {'seconds': seconds});
  }

  /// 发布音视频数据（推流），如果登录房间时 isOnlyAudio 为 YES 则只发送音频数据，视频数据不发送，推流状态的变更通过 OnPublishStateUpdate: 回调
  static void startPublishing() async {
    await _channel.invokeMethod("startPublishing");
  }

  /// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
  /// @param seconds 回调频率
  static void setPlayQualityMonitor(int seconds) async {
    await _channel.invokeMethod("setPlayQualityMonitor", {"seconds": seconds});
  }

  /// 开始播放一路音视频流，播放状态的变更通过 OnPlayStateUpdate:userId: 回调
  /// @param isZOrderMediaOverlay surfaceView的层级
  /// @param userId 要播放的音视频流用户 ID
  static void startPlayingStream(
      String userId, bool isZOrderMediaOverlay) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("startPlayingStream",
        {"streamId": userId, "isZOrderMediaOverlay": isZOrderMediaOverlay});
  }

  /// 停止播放一路音视频流
  /// @param userId 要停止播放的音视频流用户 ID
  static void stopPlayingStream(String userId) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("stopPlayingStream", {"streamId": userId});
  }

  /// 退出房间
  static void logoutRoom() async {
    await _channel.invokeMethod("logoutRoom");
  }

  /// 停止视频采集
  static void stopCapture() async {
    await _channel.invokeMethod("stopPreview");
  }

  /// 开始视频采集
  static void startCapture() async {
    await _channel.invokeMethod("startPreview");
  }

  /// 停止发布音视频数据
  static void stopPublishStream() async {
    await _channel.invokeMethod("stopPublishStream");
  }

  /// 切换摄像头
  /// @param position 摄像头位置
  static void switchCamera(bool isFrontCamera) async {
    await _channel
        .invokeMethod("switchCamera", {"isFrontCamera": isFrontCamera});
  }

  /// 设置视频编码参数
  /// @param config 视频分辨率
  /// @discussion 注：如果后面设置的视频采集分辨率大于之前设置的分辨率，摄像头可能会进行重置操作
  static void setAVConfig(LVAVConfig config) async {
    await _channel.invokeMethod("setAVConfig", {"config": config.index});
  }

  /// 静音推流
  /// @param isMute 是否静音
  static void setMuteOutput(bool isMute) async {
    await _channel.invokeMethod("setMuteOutput", {"isMute": isMute});
  }

  /// IM 鉴权是否成功
  static Future<bool> isIMAuthed() async {
    return await _channel.invokeMethod("isIMAuthed");
  }

  /// IM 是否登录成功
  static Future<bool> isIMLoginSucceed() async {
    return await _channel.invokeMethod("isIMLoginSucceed");
  }

  /// 设置美颜
  /// @param beauty 目前包含：美颜、明亮度、饱和度
  /// @param level 取值范围为[0~1]，关闭美颜传-1
  static void setBeautyLevel(LVBeauty beauty, double level) async {
    await _channel.invokeMethod(
        "setBeautyLevel", {"beauty": beauty.index, "level": level});
  }

  // MARK: - Video Callback
  static Future<dynamic> _handleVideoCallback(MethodCall call) {
    if (call.method != "onPublisherQualityUpdate" &&
        call.method != "onRemoteQualityUpdate" &&
        call.method != "onVideoSizeChanged" &&
        call.method != "onQueryIMToken") {
      print("RtmFlutterPlugin call = $call  = ${call.method}");
    }
    if (call.method == "onRemoteStreamAdd") {
      String userId = call.arguments["userId"];
      _videoCallback?.onAddRemoter(userId);
    } else if (call.method == "onRemoteStreamEnd") {
      _videoCallback?.onDeleteRemoter(call.arguments["userId"]);
    } else if (call.method == "onRoomDisconnect") {
      _videoCallback?.onRoomDisconnect(call.arguments["state"]);
    } else if (call.method == "onRoomConnected") {
      _videoCallback?.onRoomConnected();
    } else if (call.method == "onPublisherQualityUpdate") {
      LVVideoStatistic videoStatistic =
          LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPublishQualityUpdate(videoStatistic);
    } else if (call.method == "onRemoteQualityUpdate") {
      LVVideoStatistic videoStatistic =
          LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPlayQualityUpdate(
          videoStatistic, call.arguments["streamID"]);
    } else if (call.method == "onVideoSizeChanged") {
      _videoCallback?.onVideoSizeChanged(call.arguments["streamID"],
          call.arguments["width"], call.arguments["height"]);
    } else if (call.method == "onRoomMessageReceive") {
      LVIMMsg msg = LVIMMsg.convertFromMap(call.arguments);
      print("RtmFlutterPlugin call msg fromid = " +
          msg.fromID +
          "   content " +
          msg.msgContent.toString());
      _videoCallback?.onRoomMessageReceive(msg);
    } else if (call.method == "onPublishStateUpdate") {
      _videoCallback?.onPublishStateUpdate(call.arguments["state"]);
    } else if (call.method == "onPlayStateUpdate") {
      _videoCallback?.onPlayStateUpdate(
          call.arguments["state"], call.arguments["streamId"]);
    } else if (call.method == "onIMMessageReceive") {
      LVIMMsg msg = LVIMMsg.convertFromMap(call.arguments);
      print("RtmFlutterPlugin call msg fromid = " +
          msg.fromID +
          "   content " +
          msg.msgContent.toString());
      _rtmCallback?.onIMMessageReceive(msg);
    } else if (call.method == "onQueryIMToken") {
      // IM Auth
      _rtmCallback?.onQueryIMToken();
    } else if (call.method == "onIMAuthFailed") {
      int ecode = call.arguments["ecode"];
      int rcode = call.arguments["rcode"];
      String uid = call.arguments["uid"];
      bool isTokenExpired = call.arguments["isTokenExpired"];
      _rtmCallback?.onIMAuthFailed(ecode, rcode, uid, isTokenExpired);
    } else if (call.method == "onIMTokenExpired") {
      String emsg = call.arguments["emsg"];
      String token = call.arguments["token"];
      String uid = call.arguments["uid"];
      _rtmCallback?.onIMTokenExpired(uid, token, emsg);
    } else if (call.method == "onIMAuthSucceed") {
      int unReadMsgSize = call.arguments["unReadMsgSize"];
      String token = call.arguments["token"];
      String uid = call.arguments["uid"];
      _rtmCallback?.onIMAuthSucceed(uid, token, unReadMsgSize);
    }

    return Future.value(true);
  }
}
